@@ -158,7 +158,9 @@
 #include <nsIPromptService.h>
 #include <nsIWebProgressListener.h>
 #include <nsCOMPtr.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsIURI.h>
 #include <nsIURL.h>
@@ -168,6 +170,7 @@
 #include <nsIExternalHelperAppService.h>
 #include <nsCExternalHandlerService.h>
 #include <nsMemory.h>
+#include <nsNetError.h>
 
 class GContentHandler;
 struct MimeAskActionDialog;
@@ -214,7 +217,12 @@ struct MimeAskActionDialog
 /* ------------------------------------------------------------
  * Mini progress listener so that we can listen for errors
  * while downloading */
-class MiniProgressListener : public nsIWebProgressListener
+class MiniProgressListener
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+	: public nsIWebProgressListener2
+#else
+	: public nsIWebProgressListener
+#endif
 {
 public:
 	MiniProgressListener(MimeAskActionDialog *dialog) : 
@@ -226,11 +234,20 @@ public:
 
  	NS_DECL_ISUPPORTS
 	NS_DECL_NSIWEBPROGRESSLISTENER
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+	NS_DECL_NSIWEBPROGRESSLISTENER2
+#endif
 	
 	MimeAskActionDialog *mDialog;
 };
 
-NS_IMPL_ISUPPORTS1(MiniProgressListener, nsIWebProgressListener)
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+NS_IMPL_ISUPPORTS2(MiniProgressListener, nsIWebProgressListener2,
+		   nsIWebProgressListener)
+#else
+NS_IMPL_ISUPPORTS1(MiniProgressListener,
+		   nsIWebProgressListener)
+#endif
 
 // ------------------------------------------------------------
 
@@ -256,7 +273,11 @@ GContentHandler::~GContentHandler()
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 				    nsISupports *aContext,
+#ifdef HAVE_NSIHELPERAPPLAUNCHERDIALOG_SHOW_REASON
+				    PRUint32 aReason)
+#else
 				    PRBool aForced)
+#endif
 {
 	/* aForced reflects if the content being sent is normally viewable
 	 * in mozilla or not. That fact doesn't affect us, so ignore it
@@ -346,7 +367,7 @@ NS_METHOD GContentHandler::LaunchExternalDownloader (void)
 
 NS_METHOD GContentHandler::FindHelperApp (void)
 {
-	if (mUrlHelper)
+	if (mUrlHelper || mScheme.Equals("file"))
 	{
 		return LaunchHelperApp ();
 	}
@@ -378,22 +399,22 @@ NS_METHOD GContentHandler::LaunchHelperApp (void)
 			appLauncher->DeleteTemporaryFileOnExit(mTempFile);
 		}
 
-		GulString uFileName;
-		mTempFile->GetPath(uFileName);
+		GulCString filename;
+		mUri->GetPath(filename);
 
 		/* FIXME: check return value */
 		gul_general_launch_application (mHelperApp,
 						mUrlHelper ? mUrl.get()
-						: GulCString (uFileName).get(),
+							   : filename.get(),
 						gtk_get_current_event_time());
 
-		if(mUrlHelper) mLauncher->Cancel();
-	}
-	else
-	{
-		mLauncher->Cancel ();
 	}
 
+#ifdef HAVE_NSICANCELABLE_H
+	mLauncher->Cancel (NS_BINDING_ABORTED);
+#else
+	mLauncher->Cancel ();
+#endif
 	return NS_OK;
 }
 
@@ -558,7 +579,11 @@ NS_METHOD GContentHandler::MIMEAskAction (void)
 	 * errors while the download is happening to a temporary file
 	 * - bug 148739
 	 */
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+	nsCOMPtr<nsIWebProgressListener2> listener = new MiniProgressListener( dialog );
+#else
 	nsCOMPtr<nsIWebProgressListener> listener = new MiniProgressListener( dialog );
+#endif
 	mLauncher->SetWebProgressListener( listener );
 
 	return NS_OK;
@@ -756,7 +781,11 @@ mime_ask_dialog_save_clicked_cb(GtkButton *button, MimeAskActionDialog *dialog)
 	}
 	else
 	{
+#ifdef HAVE_NSICANCELABLE_H
+		launcher->Cancel(NS_BINDING_ABORTED);
+#else
 		launcher->Cancel();
+#endif
 		dialog->mContentHandler->LaunchExternalDownloader();
 	}
 	
@@ -770,7 +799,11 @@ mime_ask_dialog_download_cancel (MimeAskActionDialog *dialog)
 	dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
 
 	launcher->SetWebProgressListener( nsnull );
+#ifdef HAVE_NSICANCELABLE_H
+	launcher->Cancel(NS_BINDING_ABORTED);
+#else
 	launcher->Cancel();
+#endif
 
 	delete dialog;
 }
@@ -839,7 +872,11 @@ mime_ask_dialog_close_cb(GtkWidget *dialogWidget, MimeAskActionDialog *dialog)
 	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
 
 	launcher->SetWebProgressListener( nsnull );
+#ifdef HAVE_NSICANCELABLE_H
+	launcher->Cancel(NS_BINDING_ABORTED);
+#else
 	launcher->Cancel();
+#endif
 }
 
 // ------------------------------------------------------------
@@ -899,6 +936,13 @@ NS_IMETHODIMP MiniProgressListener::OnSecurityChange(nsIWebProgress *aWebProgres
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-
-
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+NS_IMETHODIMP MiniProgressListener::OnProgressChange64(nsIWebProgress *aWebProgress,
+						nsIRequest *aRequest, PRInt64 aCurSelfProgress,
+						PRInt64 aMaxSelfProgress, PRInt64 aCurTotalProgress,
+						PRInt64 aMaxTotalProgress)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+#endif
 
