@@ -24,7 +24,9 @@
 #include <nsIURI.h>
 #include <nsIDOMWindow.h>
 #include <nsIWindowWatcher.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsMemory.h>
 
 #include <libgnome/gnome-exec.h>
