@@ -62,7 +62,9 @@
 #include <nsIDOMNSEvent.h>
 #include <nsIDOM3Document.h>
 #include <nsIScriptSecurityManager.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsIContentPolicy.h>
 #include <nsIDOMDocumentView.h>
 #include <nsIDOMAbstractView.h>
@@ -201,6 +203,13 @@ nsresult GaleonWrapper::Init (GtkMozEmbed *mozembed)
 
 
 #ifdef HAVE_MOZILLA_PSM
+#ifdef HAVE_NSIDOCSHELL_GETSECUREUI
+	nsCOMPtr<nsIDocShell> docShell (do_GetInterface (mWebBrowser, &rv));
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	rv = docShell->GetSecurityUI (getter_AddRefs (mSecurityInfo));
+	NS_ENSURE_SUCCESS (rv, rv);
+#else
 	/* FIXME: mozilla sucks! nsWebBrowser already has an instance of this,
 	 * but we cannot get to it!
 	 * See https://bugzilla.mozilla.org/show_bug.cgi?id=94974,
@@ -215,6 +224,7 @@ nsresult GaleonWrapper::Init (GtkMozEmbed *mozembed)
 	{
 		g_warning ("Failed to instantiate nsISecureBrowserUI!\n");
 	}
+#endif
 #endif
 
 	return NS_OK;
