@@ -35,7 +35,9 @@
 
 #include <nsCOMPtr.h>
 #include <nsMemory.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsIInterfaceRequestor.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsIX509Cert.h>
@@ -46,12 +48,10 @@
 #include <nsICRLInfo.h>
 #include <nsISimpleEnumerator.h>
 #include <nsIArray.h>
-#define MOZILLA_STRICT_API 1
+#define MOZILLA_STRICT_API
 #include <nsEmbedString.h>
 #undef MOZILLA_STRICT_API
 #include <nsIDOMWindow.h>
-#include <nsIServiceManager.h>
-
 #include <gconf/gconf-client.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkstock.h>
