@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#define MOZILLA_INTERNAL_API
 /***** BEGIN UNSAFE HEADERS ******/
 #include <nsIPrintOptions.h>
 #include <nsIDocument.h>
@@ -35,6 +36,7 @@
 /***** END UNSAFE HEADERS ******/
 
 #include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsISupportsPrimitives.h>
 #include <nsIInterfaceRequestorUtils.h>
 #include <nsIPrintSettingsService.h>
@@ -132,7 +134,11 @@ MozillaPrivate::LoadOverrideStyleSheet (nsIDocShell *aDocShell, nsIURI *aUri,
 #endif
 	NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
+#if HAVE_NSIDOCUMENT_CSSLOADER
+	nsICSSLoader *loader = doc->CSSLoader();
+#else
 	nsICSSLoader *loader = doc->GetCSSLoader();
+#endif
 	NS_ENSURE_TRUE (loader, NS_ERROR_FAILURE);
 
 	/* load sheet */
