@@ -34,6 +34,9 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
 
+#define MOZILLA_INTERNAL_API
+#include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsIWebBrowserPersist.h>
 #include <nsCWebBrowserPersist.h>
 #include <nsIFileURL.h>
@@ -46,11 +49,16 @@
 #include <nsCExternalHandlerService.h>
 #include <nsMemory.h>
 
-static const PRInt32 kInterval            = 500000;     /* in microsecs == 500ms == 0.5s */
-static const PRInt32 kUnknownSizeInterval = 150000;     /* in microsecs == 150ms == 0.15s */
+static const PRInt64 kInterval            = 500000;     /* in microsecs == 500ms == 0.5s */
+static const PRInt64 kUnknownSizeInterval = 150000;     /* in microsecs == 150ms == 0.15s */
 
 #ifdef HAVE_NSITRANSFER_H
-NS_IMPL_ISUPPORTS3(GProgressListener, nsIWebProgressListener, nsITransfer, nsISupportsWeakReference)
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+NS_IMPL_ISUPPORTS4(GProgressListener, nsIWebProgressListener, nsIWebProgressListener2,
+#else
+NS_IMPL_ISUPPORTS3(GProgressListener, nsIWebProgressListener,
+#endif
+		   nsITransfer, nsISupportsWeakReference)
 #else
 NS_IMPL_ISUPPORTS4(GProgressListener, nsIWebProgressListener, 
 		   nsIDownload, nsITransfer, nsISupportsWeakReference)
@@ -78,11 +86,16 @@ download_pause_cb (Download *download, GProgressListener *self)
 
 //---------------------------------------------------------------------------
 
-GProgressListener::GProgressListener () : mPersist(nsnull),
+GProgressListener::GProgressListener () :
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+					  mCancelable(nsnull),
+#else
+					  mPersist(nsnull),
+					  mObserver(nsnull),
+#endif
 					  mGaleonPersist(nsnull),
 					  mDownload(nsnull),
 					  mContentLength(-1),
-					  mObserver(nsnull),
 					  mMIMEInfo(nsnull)
 {
 	LOG ("GProgressListener ctor(%p)", this);
@@ -101,7 +114,14 @@ NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
 	/* fill in download details */
 	mUri = aURI;
 	mFile = aFile;
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+	mCancelable = aPersist;
+#else
 	mPersist = aPersist;
+	/* pick up progress messages */
+	mPersist->SetProgressListener (this);
+#endif
+
 	mGaleonPersist = (GaleonEmbedPersist *) g_object_ref (galeonPersist);
 
 	mStartTime = PR_Now();
@@ -109,8 +129,6 @@ NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
 	/* do remaining init */
 	nsresult rv = PrivateInit ();
 
-	/* pick up progress messages */
-	mPersist->SetProgressListener (this);
 
 	/* done */
 	return rv;
@@ -175,14 +193,26 @@ NS_METHOD GProgressListener::PrivateInit (void)
 
 NS_IMETHODIMP GProgressListener::Init(nsIURI *aSource,
 				      nsIURI *aTarget,
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+				      const nsAString &aDisplayName,
+#else
                                       const PRUnichar *aDisplayName,
+#endif
 				      nsIMIMEInfo *aMIMEInfo,
                                       PRInt64 aStartTime,
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+				      nsICancelable *aCancelable)
+#else
                                       nsIWebBrowserPersist *aPersist)
+#endif
 {
         mUri = aSource;
         mStartTime = aStartTime;
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+	mCancelable = aCancelable;
+#else
         mPersist = aPersist;
+#endif
 
 	mTarget = aTarget;
 	nsCOMPtr<nsIFileURL> tFileUrl = do_QueryInterface(mTarget);
@@ -292,6 +322,7 @@ NS_IMETHODIMP GProgressListener::GetPersist(nsIWebBrowserPersist * *aPersist)
 
 #endif
 
+#ifndef HAVE_NSITRANSFER_NSICANCELABLE
 /* attribute nsIObserver observer; */
 NS_IMETHODIMP GProgressListener::GetObserver(nsIObserver * *aObserver)
 {
@@ -303,8 +334,10 @@ NS_IMETHODIMP GProgressListener::SetObserver(nsIObserver * aObserver)
         mObserver = aObserver;
         return NS_OK;
 }
+#endif
 
 /* attribute nsIWebProgressListener listener; */
+#ifndef HAVE_NSIWEBPROGRESSLISTENER2_H
 NS_IMETHODIMP GProgressListener::GetListener(nsIWebProgressListener * *aListener)
 {
         *aListener = nsnull;
@@ -314,6 +347,7 @@ NS_IMETHODIMP GProgressListener::SetListener(nsIWebProgressListener * aListener)
 {
         return NS_OK;
 }
+#endif
 
 static void LaunchHandler (GnomeVFSMimeApplication *aHandler, nsIFile *aFile,
 			   guint32 user_time)
@@ -380,11 +414,15 @@ NS_IMETHODIMP GProgressListener::OnStateChange (nsIWebProgress *aWebProgress,
                 // so kungfu is needed to survive just long enough to do the
                 // final clean up.
                 nsCOMPtr<nsITransfer> kungFuDeathGrip(this);
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+		mCancelable = nsnull;
+#else
                 if (mPersist)
                 {
                         mPersist->SetProgressListener(nsnull);
                         mPersist = nsnull;
                 }
+#endif
 		
 		if (NS_SUCCEEDED(aStatus))
 		{
@@ -486,6 +524,31 @@ NS_IMETHODIMP GProgressListener::
 					  PRInt32 aMaxSelfProgress,
 					  PRInt32 aCurTotalProgress,
 					  PRInt32 aMaxTotalProgress)
+{
+	return OnProgressChange64 (aWebProgress, aRequest, aCurSelfProgress,
+				   aMaxSelfProgress, aCurTotalProgress,
+				   aMaxTotalProgress);
+}
+
+/*
+ * void onProgressChange64 (in nsIWebProgress aWebProgress, 
+ *                          in nsIRequest aRequest, 
+ *                          in long long aCurSelfProgress, 
+ *                          in long long aMaxSelfProgress, 
+ *                          in long long aCurTotalProgress, 
+ *                          in long long aMaxTotalProgress); 
+ */
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+NS_IMETHODIMP GProgressListener::
+#else
+NS_METHOD GProgressListener::
+#endif
+			OnProgressChange64 (nsIWebProgress *aWebProgress,
+					    nsIRequest *aRequest,
+					    PRInt64 aCurSelfProgress,
+					    PRInt64 aMaxSelfProgress,
+					    PRInt64 aCurTotalProgress,
+					    PRInt64 aMaxTotalProgress)
 {
 	// no point in calculating progress if there's no one watching
 	if (!mDownload) return NS_OK;
@@ -514,8 +577,8 @@ NS_IMETHODIMP GProgressListener::
 		// pick a file name, and so on...)
 		//
 		// So lacking other reliable information, let's try the file
-		// size.  <insert 32-bits for file size ought to be enough for
-		// everyone joke here>
+		// size. Now that mozilla has clarified the api regarding large
+		// (over 32bit size) files, there's no need for a joke here. :-)
 		PRInt64 fileSize;
 		if (NS_SUCCEEDED(mFile->GetFileSize(&fileSize)))
 		{
@@ -534,7 +597,7 @@ NS_IMETHODIMP GProgressListener::
 	/* get out if we're updating too quickly, but don't miss EOF */
 	if (aCurTotalProgress != aMaxTotalProgress)
 	{
-		PRInt32 interval = (aMaxTotalProgress == -1) ? kUnknownSizeInterval : kInterval;
+		PRInt64 interval = (aMaxTotalProgress == -1) ? kUnknownSizeInterval : kInterval;
 		if ((now - mLastUpdate) < interval)
 		{
 			return NS_OK;
@@ -547,7 +610,7 @@ NS_IMETHODIMP GProgressListener::
 	guint elapsed = (now - mStartTime) / PR_USEC_PER_SEC;
 
 	/* compute download rate */
-	guint speed = 0;
+	guint64 speed = 0;
 	if (elapsed > 0)
 	{
 		speed = aCurTotalProgress / elapsed;
@@ -561,7 +624,7 @@ NS_IMETHODIMP GProgressListener::
 	}
 
 	if (aMaxTotalProgress < 0)
-		aMaxTotalProgress = G_MAXULONG;
+		aMaxTotalProgress = G_MAXINT64;
 
 	download_set_progress (mDownload,
 			       elapsed,
@@ -743,6 +806,19 @@ nsresult GProgressListener::Abort (void)
                 mRequest->Resume ();
         }
 
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+	if (mCancelable)
+	{
+		nsresult rv;
+		rv =  mCancelable->Cancel(NS_BINDING_ABORTED);
+
+		nsCOMPtr<nsIWebBrowserPersist> persist;
+		persist = do_QueryInterface(mCancelable);
+		if (persist) {
+		   return rv;
+		}
+	}
+#else
         if (mObserver)
         {
                 mObserver->Observe(NS_ISUPPORTS_CAST(nsITransfer*, this),
@@ -753,6 +829,7 @@ nsresult GProgressListener::Abort (void)
         {
                 return mPersist->CancelSave ();
         }
+#endif
 
 	// Working assumption is that if mPersist is set CancelSave correctly
 	// emits proper events to OnStateChange.  At the time of writing that
@@ -809,6 +886,11 @@ InitiateMozillaDownload (nsIURI *sourceURI, nsILocalFile* inDestFile,
 	rv = listener->InitForPersist (webPersist, displayURI, inDestFile, embedPersist);
 	NS_ENSURE_SUCCESS (rv, rv);
 
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+	rv = webPersist->SetProgressListener (listener);
+	NS_ENSURE_SUCCESS (rv, rv);
+#endif
+
 	PRInt32 persist_flags = nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
 	if (!domDocument && !decode)
 	{
