@@ -76,6 +76,17 @@ class GProgressListener :
  public:
  	NS_DECL_ISUPPORTS
 	NS_DECL_NSIWEBPROGRESSLISTENER
+#ifdef HAVE_NSIWEBPROGRESSLISTENER2_H
+	NS_DECL_NSIWEBPROGRESSLISTENER2
+#else
+	NS_METHOD GProgressListener::
+			OnProgressChange64 (nsIWebProgress *aWebProgress,
+					    nsIRequest *aRequest,
+					    PRInt64 aCurSelfProgress,
+					    PRInt64 aMaxSelfProgress,
+					    PRInt64 aCurTotalProgress,
+					    PRInt64 aMaxTotalProgress);
+#endif
 #ifndef HAVE_NSITRANSFER_H
 	NS_DECL_NSIDOWNLOAD
 #endif
@@ -98,7 +109,12 @@ class GProgressListener :
 	NS_METHOD LaunchHelperApp (void);
 	NS_METHOD SetRequest (nsIRequest *aRequest);
 
+#ifdef HAVE_NSITRANSFER_NSICANCELABLE
+	nsCOMPtr<nsICancelable> mCancelable;
+#else
 	nsCOMPtr<nsIWebBrowserPersist> mPersist;
+	nsCOMPtr<nsIObserver> mObserver;
+#endif
 	nsCOMPtr<nsIRequest> mRequest;
 	
 	GaleonEmbedPersist *mGaleonPersist;
@@ -115,11 +131,9 @@ class GProgressListener :
 	PRBool mAddToRecent;
 	
 	Download *mDownload;
-	PRInt32   mContentLength;
+	PRInt64   mContentLength;
 	GulCString mContentType;
 
-	nsCOMPtr<nsIObserver> mObserver;
-
 	nsCOMPtr<nsIMIMEInfo> mMIMEInfo;
 };
 
