@@ -31,7 +31,9 @@
 #include "GulString.h"
 
 #include <stddef.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsIURIChecker.h>
 #include <nsIHistoryEntry.h>
 #include <nsISHEntry.h>
@@ -47,6 +49,7 @@
 #include <nsIIOService.h>
 #include <nsIStreamListener.h>
 #include <nsIURI.h>
+#include <nsIInputStream.h>
 
 struct MozillaEmbedPersistPrivate
 {
@@ -291,6 +294,11 @@ GStreamListener::OnDataAvailable(nsIRequest *aRequest, nsISupports *aContext,
 {
 	LOG("[%p] OnDataAvailable(%d, %d)", this, aOffset, aCount);
 
+	// OnDataAvailable is not supposed to be called again after returning
+	// and error from a callback.  But it sometimes is...
+	// https://bugzilla.mozilla.org/show_bug.cgi?id=290371
+	NS_ENSURE_TRUE (mFunc, NS_ERROR_FAILURE);
+
 	PRUint32 totalRead;
 	nsresult rv;
 	rv = aInputStream->ReadSegments (GStreamListener::StreamReaderFunc,
@@ -314,7 +322,7 @@ GStreamListener::OnStopRequest(nsIRequest *aRequest, nsISupports *aContext, nsre
 {
 	LOG("[%p] OnStopRequest()", this);
 
-	if (aStatusCode == NS_OK && mFunc)
+	if (NS_SUCCEEDED(aStatusCode) && mFunc)
 	{
 		g_signal_emit_by_name (mEmbedPersist, "completed");
 	}
