@@ -44,7 +44,9 @@
 #include <nsIDOMHTMLLinkElement.h>
 #include <nsIStyleSheet.h>
 #include <nsIDOMDocument.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
+#undef MOZILLA_INTERNAL_API
 #include <nsIPrintSettings.h>
 #include <nsIDOMWindow.h>
 
