@@ -23,7 +23,6 @@
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
 
 #include "galeon-tab.h"
-#include "galeon-macros.h"
 #include "galeon-shell.h"
 #include "galeon-popup.h"
 #include "galeon-embed-utils.h"
@@ -37,6 +36,8 @@
 #include "gul-gestures.h"
 #include "galeon-embed-favicon.h"
 #include "galeon-debug.h"
+#include "bookmarks-gtk-menu.h"
+#include "galeon-marshal.h"
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -50,15 +51,19 @@
 #include <gtk/gtkstyle.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtkclipboard.h>
+#include <gtk/gtkmain.h>
 #include <string.h>
 
+#define GALEON_TAB_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), \
+				       GALEON_TYPE_TAB, GaleonTabPrivate))
+
+
 struct GaleonTabPrivate
 {
 	GList *notifiers;
-	GaleonWindow *window;
-	gboolean is_active;
 	GtkWidget *tab_icon;
 	TabLoadStatus load_status;
+	EmbedSecurityLevel security_level;
 	char *link_message;
 	char *js_message;
 	char status_message[255];
@@ -76,6 +81,28 @@ struct GaleonTabPrivate
 
 static GHashTable *gestures_actions = NULL;
 
+enum
+{
+	PROP_0,
+	PROP_TITLE,
+	PROP_ZOOM,
+	PROP_SECURITY,
+	PROP_MESSAGE,
+	PROP_PROGRESS,
+	PROP_POPUP_BLOCKED,
+	PROP_LOAD_STATUS,
+	PROP_LOCATION,
+	PROP_VISIBILITY,
+};
+
+enum
+{
+	SITE_VISITED,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void
 galeon_tab_class_init (GaleonTabClass *klass);
 static void
@@ -106,8 +133,9 @@ static void
 galeon_tab_net_stop_cb (GaleonEmbed *embed, GaleonTab *tab);
 static void
 galeon_tab_net_start_cb (GaleonEmbed *embed, GaleonTab *tab);
-static void
-galeon_tab_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
+
+static GaleonEmbed *
+galeon_tab_new_window_cb (GaleonEmbed *embed, 
 			  EmbedChromeMask chromemask, GaleonTab *tab);
 static void
 galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
@@ -134,6 +162,9 @@ static void
 galeon_tab_zoom_changed_cb (GaleonEmbed *embed, gint zoom, 
 			    GaleonTab *tab);
 static void
+galeon_tab_content_changed_cb (GaleonEmbed *embed, const gchar *url,
+			       GaleonTab *tab);
+static void
 galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab);
 
 static void
@@ -144,8 +175,8 @@ galeon_tab_size_allocate(GtkWidget *widget, GtkAllocation *allocation);
 
 static void galeon_tab_destroy (GtkObject *object);
 
-static void
-galeon_tab_parent_set (GtkWidget *widget, GtkWidget *previous_parent);
+static void 
+galeon_tab_map (GtkWidget *widget);
 
 static void
 galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTab *tab);
@@ -182,6 +213,77 @@ galeon_tab_get_type (void)
         return galeon_tab_type;
 }
 
+static void
+galeon_tab_set_property (GObject *object,
+			 guint prop_id,
+			 const GValue *value,
+			 GParamSpec *pspec)
+{
+	GaleonTab *tab = GALEON_TAB (object);
+	
+	switch (prop_id)
+
+	{
+	        case PROP_LOCATION:
+			galeon_tab_set_location (tab, g_value_get_string (value));
+			break;
+		case PROP_LOAD_STATUS:
+			galeon_tab_set_load_status (tab, g_value_get_int (value));
+			break;
+		case PROP_TITLE:
+		case PROP_ZOOM:
+		case PROP_SECURITY:
+		case PROP_MESSAGE:
+		case PROP_PROGRESS:
+		case PROP_POPUP_BLOCKED:
+		case PROP_VISIBILITY:
+			/* read only */
+			break;
+	}
+}
+
+static void
+galeon_tab_get_property (GObject *object,
+			 guint prop_id,
+			 GValue *value,
+		       GParamSpec *pspec)
+{
+	GaleonTab *tab = GALEON_TAB (object);
+
+	switch (prop_id)
+	{
+	        case PROP_LOCATION:
+			g_value_set_string (value, tab->priv->location);
+			break;
+		case PROP_TITLE:
+			g_value_set_string (value, tab->priv->title);
+			break;
+		case PROP_ZOOM:
+			g_value_set_int (value, tab->priv->zoom);
+			break;
+		case PROP_SECURITY:
+			g_value_set_int (value, tab->priv->security_level);
+			break;
+		case PROP_MESSAGE:
+			g_value_set_string (value, galeon_tab_get_status_message (tab));
+			break;
+		case PROP_PROGRESS:
+			g_value_set_int (value, tab->priv->load_percent);
+			break;
+		case PROP_LOAD_STATUS:
+			g_value_set_int (value, tab->priv->load_status);
+			break;
+		case PROP_POPUP_BLOCKED:
+			g_value_set_boolean (value, tab->priv->blocked_popup_uri ? TRUE : FALSE);
+			break;
+		case PROP_VISIBILITY:
+			g_value_set_boolean (value, tab->priv->visibility);
+			break;
+	}
+}
+
+
+
 static void
 galeon_tab_class_init (GaleonTabClass *klass)
 {
@@ -192,9 +294,102 @@ galeon_tab_class_init (GaleonTabClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	widget_class->size_allocate = galeon_tab_size_allocate;
-	widget_class->parent_set    = galeon_tab_parent_set;
+	widget_class->map           = galeon_tab_map;
 	gtk_object_class->destroy   = galeon_tab_destroy;
-	object_class->finalize = galeon_tab_finalize;
+	object_class->finalize      = galeon_tab_finalize;
+	object_class->set_property  = galeon_tab_set_property;
+	object_class->get_property  = galeon_tab_get_property;
+
+	signals[SITE_VISITED] =
+                g_signal_new ("site_visited",
+			      G_OBJECT_CLASS_TYPE (klass),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonTabClass, site_visited),
+                              NULL, NULL,
+                              galeon_marshal_VOID__STRING_STRING,
+                              G_TYPE_NONE,
+                              2,
+			      G_TYPE_STRING,
+			      G_TYPE_STRING);
+
+	g_object_class_install_property (object_class,
+					 PROP_TITLE,
+					 g_param_spec_string ("title",
+							      "Title",
+							      "The tab's title",
+							      _("Untitled"),
+							      G_PARAM_READABLE));
+	g_object_class_install_property (object_class,
+					 PROP_LOCATION,
+					 g_param_spec_string ("location",
+							      "Location",
+							      "The tab's location (as displayed in the url bar)",
+							      "",
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_ZOOM,
+					 g_param_spec_int ("zoom",
+							   "Zoom",
+							   "The tab's zoom value",
+							   10,
+							   9999,
+							   100,
+							   G_PARAM_READABLE));
+	g_object_class_install_property (object_class,
+					 PROP_SECURITY,
+					 g_param_spec_int ("security-level",
+							   "Security Level",
+							   "The tab's security level",
+							    STATE_IS_UNKNOWN,
+							    STATE_IS_SECURE_HIGH,
+							    STATE_IS_UNKNOWN,
+							    G_PARAM_READABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_SECURITY,
+					 g_param_spec_int ("load-status",
+							   "Load Status",
+							   "The status of the load",
+							    TAB_LOAD_NONE,
+							    TAB_LOAD_COMPLETED,
+							    TAB_LOAD_NONE,
+							    G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_MESSAGE,
+					 g_param_spec_string ("message",
+							      "Message",
+							      "The tab's statusbar message",
+							      NULL,
+							      G_PARAM_READABLE));
+	g_object_class_install_property (object_class,
+					 PROP_ZOOM,
+					 g_param_spec_int ("progress",
+							   "Progress",
+							   "The tab's load progress",
+							   0,
+							   100,
+							   0,
+							   G_PARAM_READABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_POPUP_BLOCKED,
+					 g_param_spec_boolean ("popup-blocked",
+						 	       "Popup Blocked",
+							       "Whether a popup windows was blocked",
+							       FALSE,
+							       G_PARAM_READABLE));
+
+	g_object_class_install_property (object_class,
+					 PROP_VISIBILITY,
+					 g_param_spec_boolean ("visibility",
+							       "Visibility",
+							       "The tab's visibility",
+							       TRUE,
+							       G_PARAM_READABLE));
+
+	g_type_class_add_private (klass, sizeof (GaleonTabPrivate));
 }
 
 
@@ -202,18 +397,43 @@ static void
 galeon_tab_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
 	GtkWidget *child;
+	GtkAllocation invalid = { -1, -1, 1, 1 };
 
-	g_return_if_fail(IS_GALEON_TAB (widget));
+	g_return_if_fail (GALEON_IS_TAB (widget));
 	
 	widget->allocation = *allocation;
 	child = GTK_BIN (widget)->child;
+	g_return_if_fail (child != NULL);
 
-	if (child && GTK_WIDGET_VISIBLE (child))
+	/* Only resize if we're mapped (bug #124558),
+	 * or if this is the initial size-allocate (bug #156854). */
+	if (GTK_WIDGET_MAPPED (child) ||
+	    memcmp (&child->allocation, &invalid, sizeof (GtkAllocation)) == 0)
 	{
 		gtk_widget_size_allocate (child, allocation);
 	}
 }
 
+static void
+galeon_tab_map (GtkWidget *widget)
+{
+	GtkWidget *child;
+
+	child = GTK_BIN (widget)->child;
+
+	/* We do this as using old versions of gtk, when switching
+	 * to a tab after a resize, it wouldn't get allocated the
+	 * size */
+	if (child &&
+	    memcmp (&child->allocation, &widget->allocation,
+		    sizeof (GtkAllocation)) != 0)
+	{
+		gtk_widget_size_allocate (child, &widget->allocation);
+	}
+
+	GTK_WIDGET_CLASS (parent_class)->map (widget);
+}
+
 static void
 galeon_tab_destroy (GtkObject *object)
 {
@@ -247,55 +467,23 @@ galeon_tab_destroy (GtkObject *object)
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
-static void
-galeon_tab_parent_set (GtkWidget *widget, GtkWidget *previous_parent)
-{
-	/* FIXME maybe we dont need to set the tab parent explicitly
-	 * anymore with this callback taking care of it */
-	
-	if (widget->parent)
-	{
-		GtkWidget *toplevel;
-		toplevel = gtk_widget_get_toplevel (widget);
-
-		GALEON_TAB (widget)->priv->window = GALEON_WINDOW (toplevel);
-	}
-	else
-	{
-		GALEON_TAB (widget)->priv->is_active = FALSE;
-	}
-
-	if (GTK_WIDGET_CLASS (parent_class)->parent_set)
-		GTK_WIDGET_CLASS (parent_class)->parent_set (widget, previous_parent);
-}
-
 static void
 galeon_tab_favicon_cb (GaleonEmbed *embed, const char *favicon_url,
 		       GaleonTab *tab)
 {
-	char *url1 = NULL, *url2 = NULL;
-	gboolean download;
-	gresult result;
+	char *url;
 
-	result = galeon_embed_get_location (embed, TRUE, FALSE, &url1);
-	if (result != G_OK) return;
+	url = galeon_embed_get_location (embed, TRUE, FALSE);
 
-	result = galeon_embed_get_location (embed, TRUE, TRUE, &url2);
-	if (result != G_OK) goto out;
-
-	download = (url1 != NULL && url2 != NULL && strcmp (url1, url2) == 0);
-	if (download)
+	if (url)
 	{
 		GaleonFaviconCache *cache;
 		cache = galeon_shell_get_favicon_cache (galeon_shell);
-		galeon_favicon_cache_insert_from_url (cache,
-				                      url1,
+		galeon_favicon_cache_insert_from_url (cache, url,
 						      favicon_url);
 	}
 
-out:
-	g_free (url1);
-	g_free (url2);
+	g_free (url);
 }
 
 
@@ -347,15 +535,12 @@ static void
 galeon_tab_init (GaleonTab *tab)
 {
 	GObject *embed;
-	GaleonEmbedShell *shell;
 	
 	LOG ("Creating tab %p", tab);
 
-        tab->priv = g_new0 (GaleonTabPrivate, 1);
+        tab->priv = GALEON_TAB_GET_PRIVATE (tab);
 
-	shell = galeon_shell_get_embed_shell (galeon_shell);
-	
-	embed = G_OBJECT (galeon_embed_new (G_OBJECT(shell)));
+	embed = G_OBJECT (galeon_embed_new ());
 
 	tab->priv->tab_icon = galeon_embed_favicon_new (GALEON_EMBED (embed));
 	g_object_ref (G_OBJECT (tab->priv->tab_icon));
@@ -371,8 +556,6 @@ galeon_tab_init (GaleonTab *tab)
 				 G_CALLBACK (tab_icon_changed_cb), 
 				 tab, 0);
 
-	tab->priv->window = NULL;
-	tab->priv->is_active = FALSE;
 	tab->priv->link_message = g_strdup ("");
 	tab->priv->js_message = g_strdup ("");
 	*tab->priv->status_message = '\0';
@@ -385,6 +568,7 @@ galeon_tab_init (GaleonTab *tab)
 	tab->priv->width = -1;
 	tab->priv->height = -1;
 	tab->priv->zoom   = 100;
+	tab->priv->visibility = FALSE;
 
 	gtk_container_add (GTK_CONTAINER (tab), GTK_WIDGET (embed));
 	gtk_widget_show (GTK_WIDGET(embed));
@@ -404,6 +588,9 @@ galeon_tab_init (GaleonTab *tab)
 	g_signal_connect_object (embed, "ge_zoom_change",
 				 GTK_SIGNAL_FUNC (galeon_tab_zoom_changed_cb),
 				 tab, 0);
+	g_signal_connect_object (embed, "ge_content_change",
+				 GTK_SIGNAL_FUNC (galeon_tab_content_changed_cb),
+				 tab, 0);
 	g_signal_connect_object (embed, "ge_progress",
 				 GTK_SIGNAL_FUNC (galeon_tab_progress_cb), 
 				 tab, 0);
@@ -455,8 +642,6 @@ galeon_tab_finalize (GObject *object)
 {
         GaleonTab *tab = GALEON_TAB (object);
 
-        g_return_if_fail (tab->priv != NULL);
-
 	galeon_notification_remove (&tab->priv->notifiers);
 
 	g_idle_remove_by_data (tab);
@@ -469,7 +654,6 @@ galeon_tab_finalize (GObject *object)
 
         g_free (tab->priv->title);
         g_free (tab->priv->location);
-        g_free (tab->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 
@@ -497,19 +681,24 @@ address_has_web_scheme (const char *address)
 /* Public functions */
 
 GaleonTab *
-galeon_tab_new ()
+galeon_tab_new (void)
 {
 	GaleonTab *tab;
 
-	tab = GALEON_TAB (g_object_new (GALEON_TAB_TYPE, NULL));
+	tab = GALEON_TAB (g_object_new (GALEON_TYPE_TAB, NULL));
 
 	return tab;
 }
 
-static void	      
+void
 galeon_tab_set_load_status (GaleonTab *tab,
 			    TabLoadStatus status)
 {
+	if (status == tab->priv->load_status)
+	{
+		return;
+	}
+
 	if (status == TAB_LOAD_COMPLETED)
 	{
 		Session *s;
@@ -517,15 +706,9 @@ galeon_tab_set_load_status (GaleonTab *tab,
         	session_save (s, SESSION_CRASHED);
 	}
 
-	if (galeon_tab_get_is_active (tab) && 
-	    status == TAB_LOAD_COMPLETED)
-	{
-		tab->priv->load_status = TAB_LOAD_NONE;
-	}
-	else
-	{
-		tab->priv->load_status = status;
-	}
+	tab->priv->load_status = status;
+
+	g_object_notify (G_OBJECT (tab), "load-status");
 }
 
 GaleonEmbed *
@@ -533,60 +716,22 @@ galeon_tab_get_embed (GaleonTab *tab)
 {
 	GtkWidget* child;
 
-	g_return_val_if_fail (IS_GALEON_TAB (tab), NULL);
+	g_return_val_if_fail (GALEON_IS_TAB (tab), NULL);
 
 	child = gtk_bin_get_child (GTK_BIN (tab));
 
 	return child ? GALEON_EMBED (child) : NULL;
 }
 
-void
-galeon_tab_set_window (GaleonTab *tab, GaleonWindow *window)
-{
-	g_return_if_fail (IS_GALEON_TAB (G_OBJECT (tab)));
-	if (window) g_return_if_fail (IS_GALEON_WINDOW (G_OBJECT (window)));
-
-	tab->priv->window = window;
-}
-
 GaleonWindow *
 galeon_tab_get_window (GaleonTab *tab)
 {
-	g_return_val_if_fail (IS_GALEON_TAB (G_OBJECT (tab)), NULL);
+	GtkWidget *toplevel;
 
-	return tab->priv->window;
-}
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (tab));
+	g_return_val_if_fail (GALEON_IS_WINDOW (toplevel), NULL);
 
-static void
-galeon_tab_update_color (GaleonTab *tab)
-{
-	galeon_window_set_tab_status (tab->priv->window, tab, 
-			              tab->priv->load_status);
-}
-
-void
-galeon_tab_set_is_active (GaleonTab *tab, gboolean is_active)
-{
-	TabLoadStatus status;
-	
-	g_return_if_fail (IS_GALEON_TAB (G_OBJECT (tab)));
-	
-	tab->priv->is_active = is_active;
-
-       	status = galeon_tab_get_load_status (tab);
-	if (status == TAB_LOAD_COMPLETED)
-	{
-		galeon_tab_set_load_status (tab, TAB_LOAD_NONE);
-	}
-	galeon_tab_update_color (tab);
-}
-
-gboolean
-galeon_tab_get_is_active (GaleonTab *tab)
-{
-	g_return_val_if_fail (IS_GALEON_TAB (G_OBJECT (tab)), FALSE);
-
-	return tab->priv->is_active;
+	return GALEON_WINDOW (toplevel);
 }
 
 gboolean
@@ -612,17 +757,20 @@ static void
 galeon_tab_set_visibility (GaleonTab *tab,
                            gboolean visible)
 {	
-	GaleonWindow *window;
+	g_return_if_fail (GALEON_IS_TAB (tab));
 
-	g_return_if_fail (IS_GALEON_TAB (tab));
+	if (visible)
+	{
+		gtk_widget_show (GTK_WIDGET (tab));
+	}
+	else
+	{
+		gtk_widget_hide (GTK_WIDGET (tab));
+	}
 
 	tab->priv->visibility = visible;
 
-	window = galeon_tab_get_window (tab);
-	g_return_if_fail (window != NULL);
-	
-	galeon_window_update_control (window, WindowVisibilityControl);
-
+	g_object_notify (G_OBJECT (tab), "visibility");
 }
 
 /* Private callbacks for embed signals */
@@ -632,16 +780,12 @@ galeon_tab_link_message_cb (GaleonEmbed *embed,
 			    const char *message,
 			    GaleonTab *tab)
 {
-	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+	g_return_if_fail (GALEON_IS_TAB (tab));
 
 	g_free (tab->priv->link_message);
 	tab->priv->link_message = g_strdup (message);
 	
-	if (tab->priv->is_active)
-	{
-		galeon_window_update_control (tab->priv->window,
-					      StatusbarMessageControl);
-	}
+	g_object_notify (G_OBJECT (tab), "message");
 }
 
 static void
@@ -649,85 +793,94 @@ galeon_tab_js_status_cb (GaleonEmbed *embed,
 			 const char *status,
 			 GaleonTab *tab)
 {
-	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+	g_return_if_fail (GALEON_IS_TAB (tab));
 	
 	g_free (tab->priv->js_message);
 	tab->priv->js_message = g_strdup (status);
-			
-	if (tab->priv->is_active 
-	    && eel_gconf_get_boolean (CONF_FILTERING_DEFAULT_STATUSBAR))
-	{
-		galeon_window_update_control (tab->priv->window,
-					      StatusbarMessageControl);
-	}
+
+	g_object_notify (G_OBJECT (tab), "message");
 }
 
 static void
 galeon_tab_location_cb (GaleonEmbed *embed, GaleonTab *tab)
 {		
-	CHECK_EMBED_TAB_WINDOW(embed, tab, tab->priv->window);
+	char *location;
 
-	if (tab->priv->location) g_free(tab->priv->location);
-	galeon_embed_get_location(embed, TRUE, FALSE, 
-				  &tab->priv->location);
+	g_return_if_fail (GALEON_IS_EMBED (embed));
+	g_return_if_fail (GALEON_IS_TAB (tab));
+
+	location = galeon_embed_get_location(embed, TRUE, FALSE);
+	galeon_tab_set_location (tab, location);
+
+
+	g_signal_emit (tab, signals[SITE_VISITED], 0, tab->priv->location, tab->priv->title);
+
+	g_free (location);
+}
+
+static void
+galeon_tab_content_changed_cb (GaleonEmbed *embed, const gchar *address,
+			       GaleonTab *tab)
+{
+	LOG ("Content changed: %s", address);
 
 	/* restore zoom level, but only for pages that are from the
 	 * Internet, i.e. not javascript, about or data pages */
-	if (address_has_web_scheme (tab->priv->location))
+	if (address_has_web_scheme (address))
 	{
 		GlobalHistory *history;
 		GaleonEmbedShell *ges;
 		gint saved_zoom, current_zoom;
-		gresult ret;
 
 		ges = galeon_shell_get_embed_shell (galeon_shell);
 		history = galeon_embed_shell_get_global_history (ges);
 
-		saved_zoom = global_history_get_host_zoom (history, tab->priv->location);
-		ret = galeon_embed_zoom_get (embed, &current_zoom);
-		if (ret == G_OK && saved_zoom > 0 && current_zoom != saved_zoom)
+		saved_zoom = global_history_get_host_zoom (history, address);
+		current_zoom = galeon_embed_zoom_get (embed);
+		if (saved_zoom > 0 && current_zoom != saved_zoom)
 		{
 			g_signal_handlers_block_by_func (embed, galeon_tab_zoom_changed_cb, tab);
 			tab->priv->zoom = saved_zoom;
-			galeon_embed_zoom_set (embed, saved_zoom, FALSE);
+			galeon_embed_zoom_set (embed, saved_zoom);
 			g_signal_handlers_unblock_by_func (embed, galeon_tab_zoom_changed_cb, tab);
 		}
 	}
-
-	if (tab->priv->is_active)
-	{
-		galeon_window_update_control(tab->priv->window, LocationControl);
-		galeon_window_update_control(tab->priv->window, PermissionControl);
-	}
-
-	galeon_window_site_visited(tab->priv->window, tab->priv->location, tab->priv->title);
 }
 
 static void
 galeon_tab_zoom_changed_cb (GaleonEmbed *embed, gint zoom, GaleonTab *tab)
 {
-	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+	char *address;
+
+	g_return_if_fail (GALEON_IS_TAB (tab));
+
 	tab->priv->zoom = zoom;
 
-	if (address_has_web_scheme (tab->priv->location))
+	address = galeon_embed_get_location (embed, TRUE, TRUE);
+	if (address_has_web_scheme (address))
 	{
 		GlobalHistory *history;
 		GaleonEmbedShell *ges;	
 
 		ges = galeon_shell_get_embed_shell (galeon_shell);
 		history = galeon_embed_shell_get_global_history (ges);
-		global_history_set_host_zoom (history, tab->priv->location, zoom);
-	} 
+		global_history_set_host_zoom (history, address, zoom);
+	}
+
+	g_free (address);
+	g_object_notify (G_OBJECT (tab), "zoom");
 }
 
 static void
-galeon_tab_title_cb (GaleonEmbed *embed, GaleonTab *tab)
+galeon_tab_set_title (GaleonTab *tab, const char* title)
 {
-	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
-
-	if (tab->priv->title) g_free (tab->priv->title);
-	galeon_embed_get_title (embed, &tab->priv->title);
+	if (tab->priv->title)
+	{
+		g_free (tab->priv->title);
+	}
 	
+	tab->priv->title = g_strdup (title);
+
 	if (*(tab->priv->title) == '\0')
 	{
 		g_free (tab->priv->title);
@@ -741,10 +894,28 @@ galeon_tab_title_cb (GaleonEmbed *embed, GaleonTab *tab)
 		}
 	}
 	
-	galeon_window_site_visited (tab->priv->window, tab->priv->location, tab->priv->title);
+	g_signal_emit (tab, signals[SITE_VISITED], 0, tab->priv->location, tab->priv->title);
 
-	galeon_window_set_tab_title (tab->priv->window, 
-			             tab, tab->priv->title);
+	g_object_notify (G_OBJECT (tab), "title");
+}
+
+
+static void
+galeon_tab_title_cb (GaleonEmbed *embed, GaleonTab *tab)
+{
+	gchar *title = NULL;
+
+	g_return_if_fail (GALEON_IS_EMBED (embed));
+	g_return_if_fail (GALEON_IS_TAB (tab));
+
+	title = galeon_embed_get_title (embed);
+
+	if (title)
+	{
+		galeon_tab_set_title (tab, title);
+	}
+
+	g_free (title);
 }
 
 static int
@@ -822,7 +993,7 @@ galeon_tab_progress_cb (GaleonEmbed *embed, const char *uri,
 			gint curprogress, gint maxprogress,
 			GaleonTab *tab)
 {
-	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+	g_return_if_fail (GALEON_IS_TAB (tab));
 
 	build_progress_message (tab->priv->status_message,
 				uri,
@@ -830,11 +1001,7 @@ galeon_tab_progress_cb (GaleonEmbed *embed, const char *uri,
 				maxprogress,
 				tab->priv->load_percent);
 	
-	if (tab->priv->is_active)
-	{
-		galeon_window_update_control (tab->priv->window,
-					      StatusbarMessageControl);
-	}
+	g_object_notify (G_OBJECT (tab), "message");
 }
 
 static void
@@ -847,7 +1014,7 @@ build_net_state_message (char *message,
 
 	host = get_host_name_from_uri (uri);
 	
-	/* IS_REQUEST and IS_NETWORK can be both set */
+	/* REQUEST and NETWORK can be both set */
 	
 	if (flags & EMBED_STATE_IS_REQUEST)
         {
@@ -907,8 +1074,7 @@ build_progress_from_requests (GaleonTab *tab, EmbedState state)
 		{
 			tab->priv->load_percent = load_percent;
 
-			galeon_window_update_control (tab->priv->window,
-						      StatusbarProgressControl);
+			g_object_notify (G_OBJECT (tab), "progress");
 		}
 	}
 }
@@ -920,21 +1086,15 @@ galeon_tab_net_start_cb (GaleonEmbed *embed, GaleonTab *tab)
 	tab->priv->cur_requests   = 0;
 	tab->priv->load_percent   = 0;
 
-	galeon_tab_set_load_status (tab, TAB_LOAD_STARTED);
-	galeon_tab_update_color (tab);
-	
 	if (tab->priv->blocked_popup_uri)
 	{
 		g_free(tab->priv->blocked_popup_uri);
 		tab->priv->blocked_popup_uri = NULL;
+
+		g_object_notify (G_OBJECT (tab), "popup-blocked");
 	}
 
-	galeon_window_update_control (tab->priv->window, SpinnerControl);
-
-	if (tab->priv->is_active)
-	{
-		galeon_window_update_control (tab->priv->window, StatusbarPopupBlockControl);
-	}
+	galeon_tab_set_load_status (tab, TAB_LOAD_STARTED);
 }
 
 
@@ -945,31 +1105,20 @@ galeon_tab_net_state_cb (GaleonEmbed *embed, const char *uri,
 	build_net_state_message (tab->priv->status_message, uri, state);
 	build_progress_from_requests (tab, state);
 	
-	if (tab->priv->is_active)
-	{
-		galeon_window_update_control (tab->priv->window,
-					      StatusbarMessageControl);
-
-	}
+	g_object_notify (G_OBJECT (tab), "message");
 }
 
 static void
 galeon_tab_net_stop_cb (GaleonEmbed *embed, GaleonTab *tab)
 {
 	tab->priv->load_percent = 0;
-			
+	g_object_notify (G_OBJECT (tab), "progress");
+
 	galeon_tab_set_load_status (tab, TAB_LOAD_COMPLETED);
-			
-	galeon_window_update_control (tab->priv->window, SpinnerControl);
-	if (tab->priv->is_active)
-	{
-		galeon_window_update_control (tab->priv->window, StatusbarProgressControl);
-	}
-	galeon_tab_update_color (tab);
 }
 
-static void
-galeon_tab_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
+static GaleonEmbed *
+galeon_tab_new_window_cb (GaleonEmbed *embed,
 			  EmbedChromeMask chromemask, GaleonTab *tab)
 {
 	GaleonTab *new_tab;
@@ -995,22 +1144,13 @@ galeon_tab_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
         galeon_window_add_tab (window, new_tab,
 			       GALEON_WINDOW_ADD_TAB_GROUPED, FALSE);
 	
-	*new_embed = galeon_tab_get_embed (new_tab);
+	return galeon_tab_get_embed (new_tab);
 }
 
 static void
 galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
 			  GaleonTab *tab)
 {
-	if (visibility)
-	{
-		gtk_widget_show (GTK_WIDGET (tab));
-	}
-	else
-	{
-		gtk_widget_hide (GTK_WIDGET (tab));
-	}
-        
 	galeon_tab_set_visibility (tab, visibility);
 }
 
@@ -1019,14 +1159,20 @@ static gboolean
 galeon_tab_destroy_brsr_idle_cb (GaleonTab *tab)
 {
 	GulNotebook *nb;
-	g_return_val_if_fail (IS_GALEON_TAB (tab), FALSE);
-	
+	GaleonWindow *window;
+
 	LOG ("galeon_tab_destroy_brsr_idle_cb");
 
+	g_return_val_if_fail (GALEON_IS_TAB (tab), FALSE);
+	
+	window = galeon_tab_get_window (tab);
+	g_return_val_if_fail (GALEON_IS_WINDOW (window), FALSE);
+
 	/* Don't use galeon_window_remove_tab here, we don't
 	 * want to run the "unsubmitted" changes code */
-	nb = galeon_window_get_notebook (tab->priv->window);
+	nb = galeon_window_get_notebook (window);
 	gul_notebook_remove_page (GUL_NOTEBOOK (nb), GTK_WIDGET (tab));
+
 	return FALSE;
 }
 
@@ -1054,13 +1200,13 @@ galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab)
 	LOG ("galeon_tab_destroy_brsr_cb");
 
 	g_idle_add_full (G_PRIORITY_HIGH_IDLE, 
-			 (GSourceFunc)galeon_tab_destroy_brsr_idle_cb, tab, 0);
+			 (GSourceFunc)galeon_tab_destroy_brsr_idle_cb, tab, NULL);
 }
 
 static gboolean
 let_me_resize_hack (gpointer data)
 {
-	g_return_val_if_fail (IS_GALEON_TAB (data), FALSE);
+	g_return_val_if_fail (GALEON_IS_TAB (data), FALSE);
 
 	gtk_widget_set_size_request (GTK_WIDGET(data), -1, -1);
 	return FALSE;
@@ -1119,6 +1265,44 @@ galeon_tab_show_embed_popup (GaleonTab *tab, GaleonEmbedEvent *event)
 	galeon_popup_show (popup, embed);
 }
 
+static void
+galeon_tab_bookmark_activated_cb (GbGtkMenu *sender,
+				  GbBookmarkEventActivated *ev,
+				  GaleonWindow *w)
+{
+	g_return_if_fail (GB_IS_GTK_MENU (sender));
+	g_return_if_fail (GALEON_IS_WINDOW (w));
+
+	galeon_window_bookmark_activate (w, ev);
+
+	g_object_unref (sender);
+}
+
+static void
+galeon_tab_show_bookmark_menu (GaleonTab *tab, int button)
+{
+	GbBookmarkSet *bs = galeon_shell_get_bookmark_set (galeon_shell);
+	GtkWidget *menu;
+	GaleonWindow *window;
+	GbGtkMenu *ggm;
+
+	window = galeon_tab_get_window (tab);
+	menu = gtk_menu_new();
+
+	ggm = gb_gtk_menu_new (bs->root, GTK_MENU_SHELL (menu));
+	gtk_object_sink (GTK_OBJECT (menu));
+	gb_gtk_menu_set_location_source (ggm, GB_LOCATION_SOURCE (window));
+	gb_gtk_menu_set_statusbar (ggm, GTK_STATUSBAR (galeon_window_get_statusbar (window)));
+	gb_gtk_menu_fill_children_submenus (ggm);
+	
+	g_signal_connect (ggm, "bookmark-activated", 
+			  G_CALLBACK (galeon_tab_bookmark_activated_cb), 
+			  window);
+
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, button,
+			gtk_get_current_event_time());
+}
+
 static gint
 galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 			       GaleonEmbedEvent *event,
@@ -1128,45 +1312,19 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 	int button;
 	EmbedEventContext context;
         
-	g_assert (IS_GALEON_EMBED_EVENT(event));
+	g_assert (GALEON_IS_EMBED_EVENT(event));
 
 	window = galeon_tab_get_window (tab);
 	g_return_val_if_fail (window != NULL, FALSE);
 
-	galeon_embed_event_get_mouse_button (event, &button);
-	galeon_embed_event_get_context (event, &context); 
+	button  = galeon_embed_event_get_mouse_button (event);
+	context = galeon_embed_event_get_context (event);
 
-	if (button == 2)
+	if (button == 3)
 	{
-		gint action = eel_gconf_get_integer (CONF_MOUSE_RIGHT_BUTTON_ACTION);
-		switch (action)
-		{
-		case 0: /* display context menu immediately */
-			galeon_tab_show_embed_popup (tab, event);
-			break;
-		case 1: /* perform gestures, show context menu on mouse up */
-		{ 
-			GulGestures *g = gul_gestures_new ();
-			guint x, y;
-			g_object_set_data_full(G_OBJECT(g), "embed_event", 
-					       g_object_ref (event), 
-					       g_object_unref);
-			
-			galeon_embed_event_get_coords (event, &x, &y);
-			gul_gestures_start (g, GTK_WIDGET (window), button, x, y);
-			
-			g_signal_connect (g, "gesture-performed", 
-					  G_CALLBACK (galeon_tab_gesture_performed_cb), tab);
-
-			g_object_unref (g);
-		}
-		break;
-		default:
-			g_warning ("Unexpected pref value for " CONF_MOUSE_RIGHT_BUTTON_ACTION "\n");
-			break;
-		}
+		/* Handled in the contextmenu callback */
 	}
-	else if (button == 1
+	else if (button == 2
 		&& !(context & EMBED_CONTEXT_LINK
 		     || context & EMBED_CONTEXT_EMAIL_LINK
 		     || context & EMBED_CONTEXT_INPUT))
@@ -1175,14 +1333,14 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		switch (action) 
 		{
 		case 0: /* bookmarks context menu */
-			g_warning ("Bookmarks context menu is not implemented yet\n");
+			galeon_tab_show_bookmark_menu (tab, button);
 			break;
 		case 1: /* paste url: handled by click_cb */
 		case 6: /* paste url in current tab: handled by click_cb */
 			break;
 		case 2: /* go back */
 			galeon_embed_go_back (embed);
-			break;
+			return TRUE;
 		case 3: /* gestures */
 		{
 			GulGestures *g = gul_gestures_new ();
@@ -1194,8 +1352,8 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 					  G_CALLBACK (galeon_tab_gesture_performed_cb), tab);
 			
 			g_object_unref (g);
+			return TRUE;
 		}
-		break;
 		case 4: /* auto scroll */
 		{
 			GaleonEmbedAutoscroller *as = galeon_embed_autoscroller_new ();
@@ -1204,8 +1362,8 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 			galeon_embed_autoscroller_set_embed (as, embed);
 			galeon_embed_autoscroller_start_scroll (as, GTK_WIDGET (window), x, y);
 			g_object_unref (as);
+			return TRUE;
 		}
-		break;
 		case 5: /* manual scroll */
 		{
 			GaleonEmbedManualScroller *ms = galeon_embed_manual_scroller_new ();
@@ -1214,14 +1372,14 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 			galeon_embed_manual_scroller_set_embed (ms, embed);
 			galeon_embed_manual_scroller_start_scroll (ms, GTK_WIDGET (window), x, y);
 			g_object_unref (ms);
+			return TRUE;
 		}
-		break;
 		default:
 			g_warning ("Unexpected pref value for " CONF_MOUSE_MIDDLE_BUTTON_ACTION "\n");
 			break;
 		}
 	}
-	
+
 	return FALSE;
 }
 
@@ -1272,24 +1430,22 @@ galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
 	guint button, modifier;
 	EmbedEventContext context;
         
-	g_assert(IS_GALEON_EMBED_EVENT(event));
+	g_assert(GALEON_IS_EMBED_EVENT(event));
 
 	window = galeon_tab_get_window(tab);
 	g_return_val_if_fail(window != NULL, FALSE);
 
-	galeon_embed_event_get_mouse_button(event, &button);
-	galeon_embed_event_get_context(event, &context);
-	galeon_embed_event_get_modifier(event, &modifier);
+	button = galeon_embed_event_get_mouse_button(event);
+	context = galeon_embed_event_get_context(event);
+	modifier = galeon_embed_event_get_modifier(event);
 
-	if (button == 0 && (context & EMBED_CONTEXT_LINK) &&
-	    !(context & EMBED_CONTEXT_INPUT) &&
+	if (button == 1 && (context & EMBED_CONTEXT_LINK) &&
 	    (modifier & GDK_SHIFT_MASK))
 	{
 		galeon_embed_utils_download_event_property (embed, event, 
 							    FALSE, NULL, "link");
 	}
-	else if (button == 0 && (context & EMBED_CONTEXT_IMAGE) &&
-		 !(context & EMBED_CONTEXT_INPUT) &&
+	else if (button == 1 && (context & EMBED_CONTEXT_IMAGE) &&
 		 (modifier & GDK_SHIFT_MASK))
 	{
 		galeon_embed_utils_save_event_property(embed, event, 
@@ -1298,22 +1454,22 @@ galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
 						       _("Save Image As..."),
 						       "image");
 	}
-	else if ((button == 0 && (context & EMBED_CONTEXT_LINK) &&
-		  !(context & EMBED_CONTEXT_INPUT) &&
+	else if ((button == 1 && (context & EMBED_CONTEXT_LINK) &&
 		  (modifier & GDK_CONTROL_MASK)) ||
-		 (button == 1 && (context & EMBED_CONTEXT_LINK)))
+		 (button == 2 && (context & EMBED_CONTEXT_LINK)))
 	{
 		GaleonNewTabFlags flags = 0;
 		const GValue *value;
 		const gchar *link;
 
-		if (button == 1)
+		if (button == 2)
 		{
 			flags = galeon_shell_modifier_flags (modifier);
 		}
-		galeon_embed_event_get_property(event, "link", &value);
+		value = galeon_embed_event_get_property(event, "link");
 		link = g_value_get_string (value);
 
+		flags |= GALEON_NEW_TAB_COPY_HISTORY;
 		if (galeon_embed_event_has_property(event, "link-can-open-in-new-tab"))
 		{
 			galeon_shell_new_tab(galeon_shell, window, tab, 
@@ -1324,7 +1480,7 @@ galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
 			galeon_embed_load_url (galeon_tab_get_embed (tab), link);
 		}
 	}
-	else if (button == 1 &&
+	else if (button == 2 &&
 		 (eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION) == 1 ||
 		  eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION) == 6) &&
 		 !(context & EMBED_CONTEXT_LINK ||
@@ -1343,7 +1499,12 @@ galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
 								      GDK_SELECTION_PRIMARY),
 					    galeon_tab_clipboard_text_received_cb, weak_ptr);
 	}
-	return FALSE;
+	else
+	{
+		return FALSE; /* Not handled, continue propgation */
+	}
+
+	return TRUE;
 }
 
 static void
@@ -1351,38 +1512,80 @@ galeon_tab_contextmenu_cb (GaleonEmbed *embed,
 		           GaleonEmbedEvent *event,
 			   GaleonTab *tab)
 {
-	galeon_tab_show_embed_popup (tab, event);
+	gint action = 0;
+	guint button;
+	GaleonWindow *window;
+
+	button = galeon_embed_event_get_mouse_button (event);
+
+	if (button)
+	{
+		action = eel_gconf_get_integer (CONF_MOUSE_RIGHT_BUTTON_ACTION);
+	}
+
+	window = galeon_tab_get_window (tab);
+	g_return_if_fail (GALEON_IS_WINDOW (window));
+
+	switch (action)
+	{
+	case 0: /* display context menu immediately */
+		galeon_tab_show_embed_popup (tab, event);
+		break;
+	case 1: /* perform gestures, show context menu on mouse up */
+	{ 
+		GulGestures *g = gul_gestures_new ();
+		guint x, y;
+		g_object_set_data_full(G_OBJECT(g), "embed_event", 
+				       g_object_ref (event), 
+				       g_object_unref);
+		
+		galeon_embed_event_get_coords (event, &x, &y);
+		gul_gestures_start (g, GTK_WIDGET (window), button, x, y);
+		
+		g_signal_connect (g, "gesture-performed", 
+				  G_CALLBACK (galeon_tab_gesture_performed_cb), tab);
+		
+		g_object_unref (g);
+		break;
+	}
+	default:
+		g_warning ("Unexpected pref value for " CONF_MOUSE_RIGHT_BUTTON_ACTION "\n");
+		break;
+	}
+}
+
+
+static void
+galeon_tab_set_security_level (GaleonTab *tab, EmbedSecurityLevel level)
+{
+	g_return_if_fail (GALEON_IS_TAB (tab));
+
+	tab->priv->security_level = level;
+
+	g_object_notify (G_OBJECT (tab), "security-level");
 }
 
 static void
 galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 			       GaleonTab *tab)
 {
-	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+	g_return_if_fail (GALEON_IS_TAB (tab));
 	
-	if (tab->priv->is_active)
-	{
-		galeon_window_update_control (tab->priv->window,
-					      StatusbarSecurityControl);
-	}
+	galeon_tab_set_security_level (tab, level);
 }
 
 static void
 galeon_tab_popup_blocked_cb(GaleonEmbed *embed, const char *uri,  
                             GaleonTab *tab)
 {
-        CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+	g_return_if_fail (GALEON_IS_TAB (tab));
 
 	if (tab->priv->blocked_popup_uri)
 		g_free(tab->priv->blocked_popup_uri);
 
 	tab->priv->blocked_popup_uri = uri ? g_strdup(uri) : NULL;
 
-        if (tab->priv->is_active)
-        {
-                galeon_window_update_control(tab->priv->window,
-                                             StatusbarPopupBlockControl);
-        }
+	g_object_notify (G_OBJECT (tab), "popup-blocked");
 }
 
 TabLoadStatus 
@@ -1460,15 +1663,21 @@ galeon_tab_clear_blocked_popup_uri(GaleonTab *tab)
 	{
 		g_free(tab->priv->blocked_popup_uri);
 		tab->priv->blocked_popup_uri = NULL;
+
+		g_object_notify (G_OBJECT (tab), "popup-blocked");
 	}
 }
 
 void 
 galeon_tab_set_location (GaleonTab *tab,
-			 char *location)
+			 const char *location)
 {
+	g_return_if_fail (GALEON_IS_TAB (tab));
+
 	if (tab->priv->location) g_free (tab->priv->location);
 	tab->priv->location = g_strdup (location);
+
+	g_object_notify (G_OBJECT (tab), "location");
 }
 
 static void
@@ -1680,6 +1889,7 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL,
 				      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
 				      GALEON_NEW_TAB_JUMP |
+				      GALEON_NEW_TAB_COPY_HISTORY |
 				      GALEON_NEW_TAB_IS_A_COPY);
 	}
 	else if (!strcmp (action, "up"))
@@ -1715,6 +1925,7 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL,
 				      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
 				      GALEON_NEW_TAB_JUMP |
+				      GALEON_NEW_TAB_COPY_HISTORY |
 				      GALEON_NEW_TAB_VIEW_SOURCE);
 	}
 	else if (!strcmp (action, "stop"))
@@ -1738,3 +1949,27 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 		/* unrecognized */
 	}
 }
+
+gboolean
+galeon_tab_is_empty (GaleonTab *tab)
+{
+	char *location;
+	gboolean is_empty = FALSE;
+	GaleonEmbed *embed;
+	
+	g_return_val_if_fail (GALEON_IS_TAB (tab), FALSE);
+
+	embed = galeon_tab_get_embed (GALEON_TAB (tab));
+	location = galeon_embed_get_location (embed, TRUE, TRUE);
+
+	if (location == NULL    || 
+	    location[0] == '\0' || 
+	    strcmp (location, "about:blank") == 0)
+	{
+		is_empty = TRUE;
+	}
+
+	g_free (location);
+
+	return is_empty;
+}
