@@ -99,30 +99,6 @@ window_cmd_file_print (GtkAction *action,
 	galeon_dialog_show (dialog);
 }
 
-void
-window_cmd_go_back (GtkAction *action, 
-		    GaleonWindow *window) 
-{
-	GaleonEmbed *embed;
-
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	galeon_embed_go_back (embed);
-}
-
-void
-window_cmd_go_up (GtkAction *action, 
-		    GaleonWindow *window) 
-{
-	GaleonEmbed *embed;
-
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	galeon_embed_go_up (embed);
-}
-
 void 
 window_cmd_file_send_to	(GtkAction *action, 
 		      	 GaleonWindow *window) 
@@ -164,36 +140,6 @@ window_cmd_file_send_to	(GtkAction *action,
 	g_free (url);
 }
 
-void
-window_cmd_go_forward (GtkAction *action, 
-		       GaleonWindow *window) 
-{
-	GaleonEmbed *embed;
-
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	galeon_embed_go_forward (embed);
-}
-
-void
-window_cmd_go_home (GtkAction *action, 
-		    GaleonWindow *window) 
-{
-	GaleonEmbed *embed;
-	char *location;
-	
-	embed = galeon_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	location = eel_gconf_get_string (CONF_GENERAL_HOMEPAGE);
-	g_return_if_fail (location != NULL);
-	
-	galeon_embed_load_url (embed, location);
-
-	g_free (location);
-}
-
 void
 window_cmd_go_myportal (GtkAction *action, 
 		        GaleonWindow *window) 
